﻿//DANI CSharp Demo Program 1
//Written by Jsoon Kim
//5.12.2011
//Ver 0.5

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO.Ports;
using System.IO;
using DANI_CSharp;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private API api;
        private String timerString; //temp variable used in timer routine
        private Int32 timerInt, timerInt2; //temp variable used in timer routine
        private Boolean recalibrateFlag; //flag that is raised when recalibrate button is pressed
        private Boolean freezeNutritionDisplay; //true when the nutrition display is frozen
        private SerialPort port; //handle to port; api is functional without handling this
        private Double weight; //weight value of last measurement

        private SR23_NUTR_DEF[] sr23_nutr_def; //Put nutrition definition in array for O(1) access.
        //The array is about 20% full, wasting some memory,
        //but nutr_def.txt is small enough that it won't hurt.
        private List<SR23_FOOD_DES> sr23_food_des;
        private List<SR23_NUT_DATA> sr23_nut_data;
        private Int32[] lsb_FoodList_to_sr23_nutr_def; //Mapping between index of lsb_FoodList to sr23_nutr_def
        private TextWriter logWriter;
        private Boolean logDisplayMode; //If true, the nutrition data will display data from the log

        public Form1()
        {
            InitializeComponent();
            Initialize();
            InitializeTab2();
            InitializeSR23();
            InitializeLogWriter();
        }

        private void Initialize()
        {
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.WindowState = FormWindowState.Maximized;
            this.lbl_Status.Text = "Program started...";
            this.btn_Disconnect.Enabled = false;
            this.api = new API(); //Use default config settings for now
            this.timer1.Start();
            this.txt_Search.KeyDown += new KeyEventHandler(txt_Search_KeyDown);
            this.recalibrateFlag = false;
            this.freezeNutritionDisplay = false;
            this.logDisplayMode = false;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.logWriter.Close();
        }

        private void InitializeSR23()
        {
            //Define some temporary variables
            TextReader txtRdr;
            String line;
            String[] decLine;
            Char[] delim = new Char[] { '^' };
            Char[] trim = new Char[] { '~' };
            Int32 nutr_no;

            try
            {
                //Load sr23_nutr_def
                this.sr23_nutr_def = new SR23_NUTR_DEF[1000];
                txtRdr = new StreamReader(Application.StartupPath + "\\NUTR_DEF.txt");
                line = txtRdr.ReadLine();
                while (line != null)
                {
                    decLine = line.Split(delim);
                    nutr_no = Int32.Parse(decLine[0].Trim(trim));
                    this.sr23_nutr_def[nutr_no] = new SR23_NUTR_DEF(
                        nutr_no, decLine[1].Trim(trim), decLine[3].Trim(trim));
                    line = txtRdr.ReadLine();
                }
                txtRdr.Close();

                //Load sr23_food_des
                this.sr23_food_des = new List<SR23_FOOD_DES>();
                txtRdr = new StreamReader(Application.StartupPath + "\\FOOD_DES.txt");
                line = txtRdr.ReadLine();
                while (line != null)
                {
                    decLine = line.Split(delim);

                    this.sr23_food_des.Add(
                        new SR23_FOOD_DES(
                            Int32.Parse(decLine[0].Trim(trim)),
                            Int32.Parse(decLine[1].Trim(trim)),
                            decLine[2].Trim(trim),
                            (decLine[8].Length > 0) ? Double.Parse(decLine[8]) : 0,
                            (decLine[10].Length > 0) ? Double.Parse(decLine[10]) : 0,
                            (decLine[11].Length > 0) ? Double.Parse(decLine[11]) : 0,
                            (decLine[12].Length > 0) ? Double.Parse(decLine[12]) : 0,
                            (decLine[13].Length > 0) ? Double.Parse(decLine[13]) : 0));
                    line = txtRdr.ReadLine();
                }
                txtRdr.Close();

                //Load sr23_nut_data
                this.sr23_nut_data = new List<SR23_NUT_DATA>();
                txtRdr = new StreamReader(Application.StartupPath + "\\NUT_DATA.txt");
                line = txtRdr.ReadLine();
                while (line != null)
                {
                    decLine = line.Split(delim);

                    this.sr23_nut_data.Add(
                        new SR23_NUT_DATA(
                            Int32.Parse(decLine[0].Trim(trim)),
                            Int32.Parse(decLine[1].Trim(trim)),
                            Double.Parse(decLine[2])));
                    line = txtRdr.ReadLine();
                }
                txtRdr.Close();

                //Copy over log data from file
                txtRdr = new StreamReader(Application.StartupPath + "\\log.txt");
                line = txtRdr.ReadLine();
                while (line != null)
                {
                    this.lsb_Log.Items.Add(line);
                    line = txtRdr.ReadLine();
                }
                txtRdr.Close();

                //Display data in list
                for (Int32 i = 0; i < this.sr23_food_des.Count; i++)
                    this.lsb_FoodList.Items.Add(this.sr23_food_des[i].Long_Desc);

                //Initialize FoodList_to_nutr_def
                this.lsb_FoodList_to_sr23_nutr_def = new Int32[this.lsb_FoodList.Items.Count];
                for (Int32 i = 0; i < this.lsb_FoodList_to_sr23_nutr_def.Length; i++)
                    this.lsb_FoodList_to_sr23_nutr_def[i] = i; //Initially, the nutr_def index is the same as the FoodList index
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void InitializeLogWriter()
        {
            try
            {
                this.logWriter = new StreamWriter(Application.StartupPath + "\\log.txt", true);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btn_Exit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btn_Connect_Click(object sender, EventArgs e)
        {
            this.lbl_Status.Text = "Attempting to connect...";
            this.btn_Connect.Enabled = false;
            this.Refresh();
            if (api.Connect() == 0)
            {
                this.lbl_Status.Text = "Connection Successful.";
                this.btn_Connect.Enabled = false;
                this.btn_Disconnect.Enabled = true;
            }
            else
            {
                this.lbl_Status.Text = "Connection Unsuccessful.";
                this.btn_Connect.Enabled = true;
                this.btn_Disconnect.Enabled = false;
            }
        }

        private void btn_Disconnect_Click(object sender, EventArgs e)
        {
            this.lbl_Status.Text = "Disconnecting...";
            if (api.Disconnect() == 0)
            {
                this.lbl_Status.Text = "Disconnected.";
                this.btn_Connect.Enabled = true;
                this.btn_Disconnect.Enabled = false;
                this.txt_Display.Text = "";
                this.weight = 0;
            }
            else
            {
                this.lbl_Status.Text = "Disconnection Unsuccessful.";
                this.btn_Connect.Enabled = false;
                this.btn_Disconnect.Enabled = true;
            }
        }

        private void btn_RecalibrateScale_Click(object sender, EventArgs e)
        {
            this.recalibrateFlag = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (this.recalibrateFlag)
            {
                api.SendRecalibrateCommand();
                this.recalibrateFlag = false;
            }
            else
            {
                try
                {
                    //Send request weight command
                    api.SendWeightReadCommand();
                }
                catch (Exception ex)
                {
                    //MessageBox.Show(ex.Message);
                }

                //Read response from last request weight command
                //timerString = api.DEBUG_ReadLine();
                api.CheckResponse(ref timerString);
                if (timerString != null)
                {
                    timerInt = timerString.IndexOf("w=");
                    timerInt2 = timerString.IndexOf("g");
                    if (timerInt == 0)
                    {
                        try
                        {
                            this.weight = Double.Parse(timerString.Substring(timerInt + 2, timerInt2 - (timerInt + 2)));
                        }
                        catch (Exception ex)
                        {
                            //MessageBox.Show(ex.Message);
                        }
                        this.txt_Display.Text = this.weight.ToString() + "g";
                        UpdateNutritionDisplay();
                    }
                }
            }
        }

        private void InitializeTab2()
        {
            this.port = api.GetSerialPort();

            String[] ports = SerialPort.GetPortNames();
            cmb_PortName.Items.Add("NONE");
            for (Int32 i = 0; i < ports.Length; i++)
                cmb_PortName.Items.Add(ports[i]);

            cmb_PortName.SelectedIndex = 0;
            for (Int32 i2 = 0; i2 < cmb_PortName.Items.Count; i2++)
            {
                if (cmb_PortName.Items[i2].ToString() == port.PortName)
                    cmb_PortName.SelectedIndex = i2;
            }

            cmb_BaudRate.Items.AddRange(new String[] {
                "2400",
                "4800",
                "9600",
                "14.4k",
                "19.2k",
                "28.8k",
                "38.4k",
                "57.6k",
                "76.8k",
                "115.2k",
                "230.4k"});

            switch (port.BaudRate)
            {
                case 2400:
                    cmb_BaudRate.SelectedIndex = 0;
                    break;
                case 4800:
                    cmb_BaudRate.SelectedIndex = 1;
                    break;
                case 9600:
                    cmb_BaudRate.SelectedIndex = 2;
                    break;
                case 14400:
                    cmb_BaudRate.SelectedIndex = 3;
                    break;
                case 19200:
                    cmb_BaudRate.SelectedIndex = 4;
                    break;
                case 28800:
                    cmb_BaudRate.SelectedIndex = 5;
                    break;
                case 38400:
                    cmb_BaudRate.SelectedIndex = 6;
                    break;
                case 57600:
                    cmb_BaudRate.SelectedIndex = 7;
                    break;
                case 76800:
                    cmb_BaudRate.SelectedIndex = 8;
                    break;
                case 115200:
                    cmb_BaudRate.SelectedIndex = 9;
                    break;
                case 230400:
                    cmb_BaudRate.SelectedIndex = 10;
                    break;
            }

            txt_DataBits.Text = port.DataBits.ToString();

            cmb_StopBits.Items.AddRange(new String[] {
                "None",
                "One",
                "OnePointFive",
                "Two"
                });

            if (port.StopBits == StopBits.None)
                cmb_StopBits.SelectedIndex = 0;
            else if (port.StopBits == StopBits.One)
                cmb_StopBits.SelectedIndex = 1;
            else if (port.StopBits == StopBits.OnePointFive)
                cmb_StopBits.SelectedIndex = 2;
            else if (port.StopBits == StopBits.Two)
                cmb_StopBits.SelectedIndex = 3;

            cmb_Parity.Items.AddRange(new String[] {
                "Even",
                "Mark",
                "Odd",
                "None",
                "Space"
                });

            if (port.Parity == Parity.Even)
                cmb_Parity.SelectedIndex = 0;
            else if (port.Parity == Parity.Mark)
                cmb_Parity.SelectedIndex = 1;
            else if (port.Parity == Parity.Odd)
                cmb_Parity.SelectedIndex = 2;
            else if (port.Parity == Parity.None)
                cmb_Parity.SelectedIndex = 3;
            else if (port.Parity == Parity.Space)
                cmb_Parity.SelectedIndex = 4;

            cmb_Handshake.Items.AddRange(new String[] {
                "None",
                "RequestToSend",
                "RequestToSendXOnXOff",
                "XOnXOff"});

            if (port.Handshake == Handshake.None)
                cmb_Handshake.SelectedIndex = 0;
            else if (port.Handshake == Handshake.RequestToSend)
                cmb_Handshake.SelectedIndex = 1;
            else if (port.Handshake == Handshake.RequestToSendXOnXOff)
                cmb_Handshake.SelectedIndex = 2;
            else if (port.Handshake == Handshake.XOnXOff)
                cmb_Handshake.SelectedIndex = 3;

            this.btn_OK.Enabled = false;
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            this.port.PortName = this.cmb_PortName.SelectedItem.ToString();

            switch (cmb_BaudRate.SelectedIndex)
            {
                case 0:
                    port.BaudRate = 2400;
                    break;
                case 1:
                    port.BaudRate = 4800;
                    break;
                case 2:
                    port.BaudRate = 9600;
                    break;
                case 3:
                    port.BaudRate = 14400;
                    break;
                case 4:
                    port.BaudRate = 19200;
                    break;
                case 5:
                    port.BaudRate = 28800;
                    break;
                case 6:
                    port.BaudRate = 38400;
                    break;
                case 7:
                    port.BaudRate = 57600;
                    break;
                case 8:
                    port.BaudRate = 76800;
                    break;
                case 9:
                    port.BaudRate = 115200;
                    break;
                case 10:
                    port.BaudRate = 230400;
                    break;
            }

            try
            {
                port.DataBits = Convert.ToInt32(txt_DataBits.Text);
            }
            catch (FormatException ex)
            {
                MessageBox.Show("Data bits not a valid value.");
            }

            switch (cmb_StopBits.SelectedIndex)
            {
                case 0:
                    port.StopBits = StopBits.None;
                    break;
                case 1:
                    port.StopBits = StopBits.One;
                    break;
                case 2:
                    port.StopBits = StopBits.OnePointFive;
                    break;
                case 3:
                    port.StopBits = StopBits.Two;
                    break;
            }

            switch (cmb_Parity.SelectedIndex)
            {
                case 0:
                    port.Parity = Parity.Even;
                    break;
                case 1:
                    port.Parity = Parity.Mark;
                    break;
                case 2:
                    port.Parity = Parity.Odd;
                    break;
                case 3:
                    port.Parity = Parity.None;
                    break;
                case 4:
                    port.Parity = Parity.Space;
                    break;
            }

            switch (cmb_Handshake.SelectedIndex)
            {
                case 0:
                    port.Handshake = Handshake.None;
                    break;
                case 1:
                    port.Handshake = Handshake.RequestToSend;
                    break;
                case 2:
                    port.Handshake = Handshake.RequestToSendXOnXOff;
                    break;
                case 3:
                    port.Handshake = Handshake.XOnXOff;
                    break;
            }

            this.btn_OK.Enabled = false;
            //this.Close();
        }

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            //this.Close();
        }

        private void lsb_FoodList_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.logDisplayMode = false;
            if (!freezeNutritionDisplay)
                UpdateNutritionDisplay();
        }

        private void btn_FreezeUnfreezeDisplay_Click(object sender, EventArgs e)
        {
            if (freezeNutritionDisplay)
            {
                freezeNutritionDisplay = false;
                this.btn_FreezeUnfreezeDisplay.Text = "Freeze Nutrition Data";
            }
            else
            {
                freezeNutritionDisplay = true;
                this.btn_FreezeUnfreezeDisplay.Text = "Unfreeze Nutrition Data";
            }
        }

        private void bt_AddToLog_Click(object sender, EventArgs e)
        {
            //Extract the name, weight, and food_des index of currently displaying item, delimit with tabs
            String temp = this.txt_Name.Text + "\t" + this.weight.ToString() + "\t" +
                this.lsb_FoodList_to_sr23_nutr_def[this.lsb_FoodList.SelectedIndex];
            this.lsb_Log.Items.Add(temp);
            this.logWriter.WriteLine(temp);
        }

        private void bt_ClearLog_Click(object sender, EventArgs e)
        {
            this.lsb_Log.Items.Clear();
            this.logWriter.Close();
            this.logWriter = new StreamWriter(Application.StartupPath + "\\log.txt", false);
        }

        private void lsb_Log_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.logDisplayMode = true;
            UpdateNutritionDisplay();
        }

        private void UpdateNutritionDisplay()
        {
            Int32 ndb_no, food_des_index = 0;
            Double serving_weight = 0;
            Int32 curIndex, lowBound, highBound;

            //No items selected yet
            if (this.lsb_FoodList.SelectedIndex < 0 && this.lsb_Log.SelectedIndex < 0)
                return;
            else if (this.lsb_FoodList.SelectedIndex >= 0 && !logDisplayMode)
            {
                serving_weight = this.weight;
                food_des_index = this.lsb_FoodList_to_sr23_nutr_def[this.lsb_FoodList.SelectedIndex];
            }
            else if (this.lsb_Log.SelectedIndex >= 0 && logDisplayMode)
            {
                String temp = this.lsb_Log.Items[this.lsb_Log.SelectedIndex].ToString();
                Int32 delim_Index1 = temp.IndexOf("\t");
                Int32 delim_Index2 = temp.IndexOf("\t", delim_Index1 + 1);
                try
                {
                    serving_weight = Double.Parse(temp.Substring(delim_Index1 + 1, delim_Index2 - delim_Index1 - 1));
                    food_des_index = Int32.Parse(temp.Substring(delim_Index2 + 1, temp.Length - delim_Index2 - 1));
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                    return;
                }
            }

            ndb_no = this.sr23_food_des[food_des_index].NDB_No;

            //Find first index of ndb_no
            lowBound = 0;
            highBound = this.sr23_nut_data.Count;
            curIndex = (highBound - lowBound) / 2;
            while (true)
            {
                if (this.sr23_nut_data[curIndex].NDB_No == ndb_no)
                {
                    //Check if this index really is the first index
                    while (curIndex != 0 &&
                        this.sr23_nut_data[curIndex - 1].NDB_No == ndb_no)
                    {
                        curIndex = curIndex - 1;
                    }
                    break;
                }
                else if (this.sr23_nut_data[curIndex].NDB_No > ndb_no)
                {
                    //Check for item not found
                    if (curIndex == lowBound)
                    {
                        curIndex = -1;
                        break;
                    }
                    highBound = curIndex;
                    curIndex = lowBound + (highBound - lowBound) / 2;
                }
                else
                {
                    //Check for item not found
                    if (curIndex == lowBound)
                    {
                        curIndex = -1;
                        break;
                    }
                    lowBound = curIndex;
                    curIndex = lowBound + (highBound - lowBound) / 2;
                }
            }

            //If first index is found, spit data to text display
            if (curIndex != -1)
            {
                this.txt_Name.Text = this.sr23_food_des[food_des_index].Long_Desc;
                this.lbl_ServingSize_g.Text = serving_weight.ToString() + "g";

                while (curIndex < this.sr23_nut_data.Count &&
                    this.sr23_nut_data[curIndex].NDB_No == ndb_no)
                {
                    if (this.sr23_nut_data[curIndex].Nutr_No == 208) // Calories
                    {
                        this.lbl_Calories_kcal.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString();
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 204) // Total Fat
                    {
                        this.lbl_TotalFat_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                        this.lbl_TotalFat_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 65)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 606) // Saturated Fat
                    {
                        this.lbl_SaturatedFat_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                        this.lbl_SaturatedFat_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 20)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 645) // Monounsaturated Fat
                    {
                        this.lbl_MonounsaturatedFat_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 646) // Polyunsaturated Fat
                    {
                        this.lbl_PolyunsaturatedFat_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 605) // Trans Fat
                    {
                        this.lbl_TransFat_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 601) // Cholesterol
                    {
                        this.lbl_Cholesterol_mg.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "mg";
                        this.lbl_Cholesterol_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 300)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 307) // Sodium
                    {
                        this.lbl_Sodium_mg.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "mg";
                        this.lbl_Sodium_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 2400)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 306) // Potassium
                    {
                        this.lbl_Potassium_mg.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "mg";
                        this.lbl_Potassium_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 3500)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 205) // Total Carbohydrate
                    {
                        this.lbl_TotalCarbohydrate_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                        this.lbl_TotalCarbohydrate_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 300)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 291) // Fiber
                    {
                        this.lbl_DietaryFiber_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                        this.lbl_DietaryFiber_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 25)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 269) // Sugars
                    {
                        this.lbl_Sugars_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 203) // Protein
                    {
                        this.lbl_Protein_g.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 100)).ToString() + "g";
                        this.lbl_Protein_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 50)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 318) // Vitamin A
                    {
                        this.lbl_VitaminA_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 5000)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 401) // Vitamin C
                    {
                        this.lbl_VitaminC_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 60)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 301) // Calcium
                    {
                        this.lbl_Calcium_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 1000)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 303) // Iron
                    {
                        this.lbl_Iron_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 18)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 324) // Vitamin D
                    {
                        this.lbl_VitaminD_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 400)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 323) // Vitamin E
                    {
                        this.lbl_VitaminE_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 30)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 430) // Vitamin K
                    {
                        this.lbl_VitaminK_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 80)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 404) // Thiamin
                    {
                        this.lbl_Thiamin_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 1.5)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 405) // Riboflavin
                    {
                        this.lbl_Riboflavin_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 1.7)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 406) // Niacin
                    {
                        this.lbl_Niacin_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 20)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 415) // Vitamin B6
                    {
                        this.lbl_VitaminB6_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 2.0)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 417) // Folate
                    {
                        this.lbl_Folate_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 400)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 418) // Vitamin B12
                    {
                        this.lbl_VitaminB12_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 6.0)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 305) // Phosphorus
                    {
                        this.lbl_Phosphorus_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 1000)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 304) // Magnesium
                    {
                        this.lbl_Magnesium_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 400)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 309) // Zinc
                    {
                        this.lbl_Zinc_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 15)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 317) // Selenium
                    {
                        this.lbl_Selenium_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 70)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 312) // Copper
                    {
                        this.lbl_Copper_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 2.0)).ToString() + "%";
                    }
                    else if (this.sr23_nut_data[curIndex].Nutr_No == 315) // Manganese
                    {
                        this.lbl_Manganese_pct.Text = ((Int32)(this.sr23_nut_data[curIndex].Nutr_Val * serving_weight / 2.0)).ToString() + "%";
                    }

                    ////Nutrients Detailed List
                    //this.rtb_Nutrients.Text += this.sr23_nutr_def[this.sr23_nut_data[curIndex].Nutr_No].NutrDesc + ": "
                    //    + (this.sr23_nut_data[curIndex].Nutr_Val * this.weight / 100).ToString() + " "
                    //    + this.sr23_nutr_def[this.sr23_nut_data[curIndex].Nutr_No].Units + "\n";
                    curIndex++;
                }
            }
        }

        private void bt_Search_Click(object sender, EventArgs e)
        {
            SearchFoodData();
        }

        private void txt_Search_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                SearchFoodData();
            }
        }

        private void SearchFoodData()
        {
            //Reload data in list
            this.lsb_FoodList.Items.Clear();
            for (Int32 i = 0; i < this.sr23_food_des.Count; i++)
                this.lsb_FoodList.Items.Add(this.sr23_food_des[i].Long_Desc);

            //Remove all list items that do not match the search term
            Int32 itemsRemoved = 0;
            for (Int32 i = 0; i < this.sr23_food_des.Count; i++)
            {
                if (this.sr23_food_des[i].Long_Desc.ToLower().Contains(this.txt_Search.Text.ToLower()))
                {
                    this.lsb_FoodList_to_sr23_nutr_def[i - itemsRemoved] = i;
                }
                else
                {
                    this.lsb_FoodList.Items.RemoveAt(i - itemsRemoved);
                    itemsRemoved++;
                }
            }

            return;

            //Int32 curIndex, firstIndex = this.lsb_FoodList.SelectedIndex + 1;
            //if (firstIndex == this.sr23_food_des.Count || firstIndex == -1)
            //    curIndex = 0;
            //else
            //    curIndex = firstIndex;
            //for (Int32 i = firstIndex; i < this.sr23_food_des.Count; i++)
            //{
            //    if (this.sr23_food_des[i].Long_Desc.ToLower().Contains(this.txt_Search.Text.ToLower()))
            //    {
            //        this.lsb_FoodList.SelectedIndex = i;
            //        return;
            //    }
            //}
            //for (Int32 i = 0; i < curIndex; i++)
            //{
            //    if (this.sr23_food_des[i].Long_Desc.ToLower().Contains(this.txt_Search.Text.ToLower()))
            //    {
            //        this.lsb_FoodList.SelectedIndex = i;
            //        return;
            //    }
            //}
            //MessageBox.Show("Search term not found.");
            //return;
        }

        private void btn_DisplayAllItems_Click(object sender, EventArgs e)
        {
            //Reload data in list
            this.lsb_FoodList.Items.Clear();
            for (Int32 i = 0; i < this.sr23_food_des.Count; i++)
                this.lsb_FoodList.Items.Add(this.sr23_food_des[i].Long_Desc);
        }

        private void cmb_PortName_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }

        private void cmb_BaudRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }

        private void txt_DataBits_TextChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }

        private void cmb_StopBits_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }

        private void cmb_Parity_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }

        private void cmb_Handshake_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.btn_OK.Enabled = true;
        }
    }
}
