//Written by Jsoon Kim
//Updated 5.10.11

// Mega644 version

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>

//String ops
#include <string.h>

//set up the debugging utility ASSERT
//#define __ASSERT_USE_STDERR
//#include <assert.h>

// serial communication library
//#include "uart.h"

#define begin {
#define end }
#define t1 1
 
void initialize(void); //all the usual mcu stuff 
void main_routine(void); //main loop routine
void USART_Transmit( unsigned char data ); //transmit one char over USART
unsigned char USART_Receive(void); //receive one char over USART
//32-bit adddition
void THIRTY_TWO_BIT_UNSIGNED_PLUS(unsigned char* a3,
					unsigned char* a2,
					unsigned char* a1,
					unsigned char* a0,
					unsigned char* b3,
					unsigned char* b2,
					unsigned char* b1,
					unsigned char* b0,
					unsigned char* o3,
					unsigned char* o2,
					unsigned char* o1,
					unsigned char* o0);

char str[80]; //buffer for string to transmit over USART
int strlength; //length of string in buffer
unsigned char recv_temp; //temporary storage for received character
unsigned char recv[2]; //currently received char in USART
unsigned int calibration; //calibration offset
volatile unsigned char time1;
unsigned int weight;
unsigned int weight_temp;
unsigned long long oversample_counter;
//unsigned int oversample_accum;
unsigned char oversample_accum_3;
unsigned char oversample_accum_2;
unsigned char oversample_accum_1;
unsigned char oversample_accum_0;
unsigned char empty_char;
unsigned int debug;
unsigned int overflow_detection; //used in adding 32 bit numbers
unsigned char temp_a3; //temporary storage
unsigned char temp_a2; //temporary storage
unsigned char temp_a1; //temporary storage
unsigned char temp_a0; //temporary storage
unsigned char temp_b3; //temporary storage
unsigned char temp_b2; //temporary storage
unsigned char temp_b1; //temporary storage
unsigned char temp_b0; //temporary storage
unsigned char temp_ADCH; //temporary storage
unsigned char temp_ADCL; //temporary storage
//unsigned char test_a3;
//unsigned char test_a2;
//unsigned char test_a1;
//unsigned char test_a0;
//unsigned char test_b3;
//unsigned char test_b2;
//unsigned char test_b1;
//unsigned char test_b0;

const float GRAMS_PER_ADC_UNIT = 3.000; //needs calibration

//1ms ISR
ISR (TIMER0_COMPA_vect)
begin      
  //Decrement the three times if they are not already zero
  if (time1>0)	--time1;
end  

//main entry point
int main(void)
begin  
  initialize();
  
  //main task scheduler loop 
  while(1)
  begin
  	//if (time1==0){ time1=t1; main_routine();}
	main_routine();
  end
end  

//main loop routine
void main_routine(void)
begin
	//if (strlength >= strlen(str)) //Finished sending all characters
	//begin
		//Get most recent data from ADC
	  	//while (ADCSRA&(1<<ADSC)); //Stall while ADC is busy
	  	//ADCSRA |= (1<<ADSC); //Start new ADC command
	  	//while (ADCSRA&(1<<ADSC)); //Stall until ADC is finished

		//Go to sleep, wake up when ADC finished
		asm("sleep");

		if(oversample_counter == 4095)
		begin
			oversample_counter = 0;
			//weight = (oversample_accum * GRAMS_PER_ADC_UNIT) >> 4;
			//weight = oversample_accum;
			weight = (unsigned int)
				((((float)oversample_accum_0) * GRAMS_PER_ADC_UNIT +
				((float)oversample_accum_1) * GRAMS_PER_ADC_UNIT * 256.0 +
				((float)oversample_accum_2) * GRAMS_PER_ADC_UNIT * 65536.0 +
				((float)oversample_accum_3) * GRAMS_PER_ADC_UNIT * 16777216.0) / 1024.0); 
			//oversample_accum = (ADCH<<8) + ADCL;
			temp_ADCL = ADCL;
			temp_ADCH = ADCH;
			THIRTY_TWO_BIT_UNSIGNED_PLUS(
				&empty_char,
				&empty_char,
				&temp_ADCH,
				&temp_ADCL,
				&empty_char,
				&empty_char,
				&empty_char,
				&empty_char,
				&oversample_accum_3,
				&oversample_accum_2,
				&oversample_accum_1,
				&oversample_accum_0);

			//ADCL;
			//ADCH;
			
			//Initialize calibration for first time after power on, after some delay
			//if (calibration < -1)
			//begin
			//	calibration++;
			//end
			if (calibration == -1)
			begin
				calibration = weight;
			end
		end
		else
		begin
			oversample_counter++;
			//oversample_accum += (ADCH<<8) + ADCL;
			temp_ADCL = ADCL;
			temp_ADCH = ADCH;
			THIRTY_TWO_BIT_UNSIGNED_PLUS(
				&empty_char,
				&empty_char,
				&temp_ADCH,
				&temp_ADCL,
				&oversample_accum_3,
				&oversample_accum_2,
				&oversample_accum_1,
				&oversample_accum_0,
				&oversample_accum_3,
				&oversample_accum_2,
				&oversample_accum_1,
				&oversample_accum_0);

			//ADCL;
			//ADCH;
			//TEST_DEBUG
			//THIRTY_TWO_BIT_UNSIGNED_PLUS(
			//	&test_a3,
			//	&test_a2,
			//	&test_a1,
			//	&test_a0,
			//	&test_b3,
			//	&test_b2,
			//	&test_b1,
			//	&test_b0,
			//	&test_a3,
			//	&test_a2,
			//	&test_a1,
			//	&test_a0);

		end

		recv_temp = USART_Receive(); //Get received char if there exists any
		//There was an incoming character on USART
		if (recv_temp != 0)
		begin
			recv[1] = recv[0];
			recv[0] = recv_temp;
		end

		//Process commands

		//Check for "ww" input; if so, start transmission
		if (recv[1] == 'w' && recv[0] == 'w')
		begin
			//weight = ADCL;
			if (calibration > weight) //weight is negative
				weight_temp = 0;
			else
				//weight_temp = (unsigned int)(((float)weight - (float)calibration) * GRAMS_PER_ADC_UNIT) >> 8;
				weight_temp = weight - calibration;
			sprintf(str,"w=%dg w=%dg\r\n",
				(unsigned int)(weight_temp),
				(unsigned int)(weight_temp));
			//ADCH; // Read ADCH to reset ADC
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "cc" input; if so, reset calibration
		else if (recv[1] == 'c' && recv[0] == 'c')
		begin
			//calibration = ADCL; // Read ADCL to reset ADC
			calibration = weight;
			//ADCH; // Read ADCH to reset ADC
			sprintf(str,"c=%dg c=%dg\r\n",
				(unsigned int)calibration,
				(unsigned int)calibration);
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "tt" input; if so, return "Test" (test to check connection)
		else if (recv[1] == 't' && recv[0] == 't')
		begin
			sprintf(str,"test success test success\r\n");
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "vv" input; if so, return version number
		else if (recv[1] == 'v' && recv[0] == 'v')
		begin
			sprintf(str,"1h.8 1h.8");
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "11" input; if so, turn light 1 (PC1) on
		else if (recv[1] == '1' && recv[0] == '1')
		begin
			sprintf(str,"Light 1 On Light 1 On\r\n");
			PORTC = PORTC & 0xfd;
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "22" input; if so, turn light 1 (PC1) off
		else if (recv[1] == '2' && recv[0] == '2')
		begin
			sprintf(str,"Light 1 Off Light 1 Off\r\n");
			PORTC = PORTC | 0x02;
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "33" input; if so, turn light 2 (PC2) on
		else if (recv[1] == '3' && recv[0] == '3')
		begin
			sprintf(str,"Light 2 On Light 2 On\r\n");
			PORTC = PORTC & 0xfb;
			strlength = 0;
			recv[0] = 0;
		end
		//Check for "44" input; if so, turn light 2 (PC2) off
		else if (recv[1] == '4' && recv[0] == '4')
		begin
			sprintf(str,"Light 2 Off Light 2 Off\r\n");
			PORTC = PORTC | 0x04;
			strlength = 0;
			recv[0] = 0;
		end
		//Debug command
		else if (recv[1] == 'd' && recv[0] == 'd')
		begin
			debug = 0;
			//sprintf(str,"debug=%d\r\n", (unsigned int)oversample_counter);
			//sprintf(str,"debug=%d,%d,%d,%d\r\n", (unsigned int)test_a3,
			//	 (unsigned int)test_a2,  (unsigned int)test_a1,  (unsigned int)test_a0);
			sprintf(str,"debug=%d,%d,%d,%d,%d,%d\r\n", (unsigned int)oversample_accum_3,
				 (unsigned int)oversample_accum_2, 
				 (unsigned int)oversample_accum_1,
				 (unsigned int)oversample_accum_0,
				 (unsigned int)temp_ADCH,
				 (unsigned int)temp_ADCL);
			strlength = 0;
			recv[0] = 0;
		end
		else
		begin
			//ADCL; // Read ADCL to reset ADC
			//ADCH; // Read ADCH to reset ADC
		end

	//end
	//else //Transmit next unsent character
	if (strlength < strlen(str))
	begin
		USART_Transmit(str[strlength]);
		strlength++;
	end
end

void initialize(void)
begin
	//set up the ports
	DDRB=0x0f;    // PORT B is an output  
	PORTB=0;
	DDRD=0x00;	// PORT D is an input 
	DDRC=0xff; //PORT C is an output (LEDs)
	PORTC=0xff; //turn all LEDs off
           
	//set up timer 0 for 1 mSec ticks
	TIMSK0 = 2;		//turn on timer 0 cmp match ISR 
	OCR0A = 250;  	//set the compare reg to 250 time ticks
	TCCR0A = 0b00000010; // turn on clear-on-match
	TCCR0B = 0b00000011;	// clock prescalar to 64
      
	//crank up the ISRs
	sei();

	//Init USART
	// Disable PRR
	PRR = 0x00;
	// Set baud rate
	UBRR0L = (unsigned char)23;
	UBRR0H = (unsigned char)0;
	UCSR0A = 0;
	// Enable receiver and transmitter
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	// Set frame format: 8data, 1stop bit
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);

	//Init ADC
	////Set Diff input ADC1/ADC0, gain 200x, internal 2.56V reference
	//ADMUX = (1<<MUX3)|(1<<MUX1)|(1<<MUX0)|(1<<REFS1)|(1<<REFS0);
	
	//Set single ended input ADC0	
	//ADMUX = 0;

	//Internal 2.56V reference
	ADMUX = (0<<REFS1)|(1<<REFS0);
	
	//Set Diff input ADC1/ADC0, gain 10x
	//ADMUX = (1<<MUX3)|(1<<MUX0);

	//Turn ADC On, set ADC clk to XTAL/32, free running mode on
	//ADCSRA = (1<<ADEN)|(1<<ADATE)|(1<<ADPS2)|(1<<ADPS0);
	//free running mode off
	//ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS0);

	//ADC sleep interrupt on
	ADCSRA = (1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS0);
	//Set sleep to ADCNRM, sleep enabled
	//SMCR = (1<<SM0)|(1<<SE);
	//Set sleep to idle mode (allows USART to keep working), sleep enabled
	SMCR = (1<<SE);

	weight = 0;
	strlength = 0;
	calibration = -1;
	oversample_counter = 0;
	//oversample_accum = 0;
	oversample_accum_3 = 0;
	oversample_accum_2 = 0;
	oversample_accum_1 = 0;
	oversample_accum_0 = 0;
	empty_char = 0;

	//TEST_DEBUG
	//test_a3 = 0;
	//test_a2 = 0;
	//test_a1 = 0;
	//test_a0 = 0;
	//test_b3 = 0;
	//test_b2 = 0;
	//test_b1 = 0;
	//test_b0 = 1;
end  

   
void USART_Transmit( unsigned char data )
{
	// Wait for empty transmit buffer 
	while ( !( UCSR0A & (1<<UDRE0)) ) ;
	// Put data into buffer, sends the data 
	UDR0 = data;
}

unsigned char USART_Receive(void)
{
	// Wait for data to be received 
	//while ( !(UCSR0A & (1 << RXC0)) ) ;

	// Check if received data exists 
	if ( !(UCSR0A & (1 << RXC0)) )
		return 0;

	// Get and return received data from buffer 
	return UDR0;
}

//Does A + B = O
void THIRTY_TWO_BIT_UNSIGNED_PLUS(unsigned char* a3,
					unsigned char* a2,
					unsigned char* a1,
					unsigned char* a0,
					unsigned char* b3,
					unsigned char* b2,
					unsigned char* b1,
					unsigned char* b0,
					unsigned char* o3,
					unsigned char* o2,
					unsigned char* o1,
					unsigned char* o0)
{
	//Copy variables first in case O has the same address as A or B
	temp_a3 = *a3;
	temp_a2 = *a2;
	temp_a1 = *a1;
	temp_a0 = *a0;
	temp_b3 = *b3;
	temp_b2 = *b2;
	temp_b1 = *b1;
	temp_b0 = *b0;

	//First ignore all overflow
	*o0 = temp_a0 + temp_b0;
	*o1 = temp_a1 + temp_b1;
	*o2 = temp_a2 + temp_b2;
	*o3 = temp_a3 + temp_b3;

	//Then process all overflow bits
	overflow_detection = ((unsigned int)temp_a0) + ((unsigned int)temp_b0);
	if ((overflow_detection >> 8) > 0) //overflow in lowest byte
	{
		if (*o1 == 255)
		{
			*o1 = 0;
			if (*o2 == 255)
			{
				*o2 = 0;
				*o3 = *o3 + 1; //overflow may occur when 32 bits can't hold A+B
			}
			else
				*o2 = *o2 + 1;
		}
		else
			*o1 = *o1 + 1;
	}
	overflow_detection = ((unsigned int)temp_a1) + ((unsigned int)temp_b1);
	if ((overflow_detection >> 8) > 0) //overflow in second lowest byte
	{
		if (*o2 == 255)
		{
			*o2 = 0;
			*o3 = *o3 + 1; //overflow may occur when 32 bits can't hold A+B
		}
		else
			*o2 = *o2 + 1;
	}
	overflow_detection = ((unsigned int)temp_a2) + ((unsigned int)temp_b2);
	if ((overflow_detection >> 8) > 0) //overflow in third lowest byte
	{
		*o3 = *o3 + 1; //overflow may occur when 32 bits can't hold A+B
	}
}

ISR(ADC_vect)
begin
	//nothing
end
